// BlueMatikTestWin.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "BlueMatikTestWin.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#include <stdio.h>
#include <sys/timeb.h>
#include <time.h>

#define BUFFLEN 1024
#define NAMEMAXLEN 21

// static variables
CWinApp theApp;
using namespace std;
HANDLE ghOverlappedFile = NULL;	
OVERLAPPED gWriteOverlapped ;
OVERLAPPED gReadOverlapped ;

// prototypes of functins later in this file
int OpenOverlappedFile(CString& Filename);
int WriteToCom( char * lpBuffer, int nBytes = -1 );
int ReadFromCom( char * lpBuffer, DWORD iChars);
int PressAnyKeyToFinish( bool bSuccess = false );

// main program
int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	char szLine[BUFFLEN];
	DWORD NumReadCumulative = 0;
	DWORD NumRead = 0;
	unsigned short isCount, isReadVal;

	// initialize MFC 
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		_tprintf(_T("Fatal Error: MFC initialization failed\n"));
		return 1;
	}

	// announcement
	printf( "ToothPIC low-level test program\n" );
	printf( "===============================\n" );
	printf( "\n" );

	// reminder to test without BlueMatik first
	printf( "Press enter to confirm:\n\n");
	printf( "  1. The test program was loaded into ToothPIC\n\n");
	printf( "  2. If desired, ToothPIC was powered up without\n     BlueMatik and the red LED flashed rapidly\n\n");
	printf( "  3. ToothPIC is ready to be tested with BlueMatik\n     attached\n\n");
	printf( "  4. Power was applied to ToothPIC and the\n     green LED came on after about 1 second\n\n\n");
	printf( "Do not connect or disconnect BlueMatik with\npower applied.\n" );
	getchar();

	// get com ports
	long IncomingCOM = 4;
	long OutgoingCOM = 4;
	printf( "\nEnter the COM port number for incoming \n('local services') connections.\n", IncomingCOM );
	scanf( "%ld", &IncomingCOM );
	printf( "You selected COM%ld:\n\n", IncomingCOM );
	printf( "Enter the COM port number for outgoing \n('client applications') connections.\n", OutgoingCOM );
	scanf( "%ld", &OutgoingCOM );
	printf( "You selected COM%ld:\n\n", OutgoingCOM );

	// test slave mode
	printf( "To begin testing the ToothPIC, connect\nto its Bluetooth serial port service,\nwithout authenticating, if possible, from this PC.\n\nIf you have already made the connection,\ndisconnect it and start the test again\nINCLUDING restarting ToothPIC.\n\n" );
	printf( "If this device is already paired, delete\nthe pair record from the list of paired\ndevices on this computer.\n\n" );
	printf( "Waiting for you to connect.........." );

	// connect to com port
	CString ComStr;
	ComStr.Format( "\\\\.\\COM%ld", OutgoingCOM );

	if ( OpenOverlappedFile( ComStr ) )
	{
		printf( "Error can't open COM%ld:\n\n", OutgoingCOM );
		return PressAnyKeyToFinish( );
	}

	// await phrase "Slave" as indication of success
	if (ReadFromCom( szLine, 5 )) 
	{
		printf( "FAILED - bluetooth error\n" );
		return PressAnyKeyToFinish( );
	}
	if (strncmp( szLine, "Slave", 5 ) ) 
	{
		printf( "FAILED - data error\n" );
		return PressAnyKeyToFinish( );
	}

	printf( "OK\n" );

	// disconect
	printf( "Disconnecting......................." );
			
	// send phrase "Master" as signal to disconnect
	if (WriteToCom( "Master" )) return PressAnyKeyToFinish( );

	// await phrase "Disco" as indication of success
	if (ReadFromCom( szLine, 5 )) 
	{
		printf( "FAILED - bluetooth error\n" );
		return PressAnyKeyToFinish( );
	}
	if (strncmp( szLine, "Disco", 5 ) ) 
	{
		printf( "FAILED - data error\n" );
		return PressAnyKeyToFinish( );
	}

	CloseHandle( ghOverlappedFile );
	printf( "OK\n" );

	// test authenticated connection with PIN code
	// note, if previously successfully connected, PIN may be stored
	// so this test becomes a repeat link key test that follows
	printf( "The PIN code you must enter is......\"0000\"\n" );
	printf( "Connecting as authenticated slave..." );
			
	// connect to com port
	ComStr.Format( "\\\\.\\COM%ld", IncomingCOM );
	if ( OpenOverlappedFile( ComStr ) )
	{
		printf( "Error can't open COM%ld:\n\n", IncomingCOM );
		return PressAnyKeyToFinish( );
	}

	// await greeting text
	if (ReadFromCom( szLine, 10 )) 
	{
		printf( "FAILED - bluetooth error\n" );
		return PressAnyKeyToFinish( );
	}
	if (strncmp( szLine, "AuthMaster", 10 ) ) 
	{
		printf( "FAILED - data error\n" );
		return PressAnyKeyToFinish( );
	}

	printf( "OK\n" );

	// disconect
	printf( "Disconnecting......................." );
			
	// send phrase "Slave" as signal to disconnect
	if (WriteToCom( "Slave" )) return PressAnyKeyToFinish( );

	// await phrase "Disco" as indication of success
	if (ReadFromCom( szLine, 5 )) 
	{
		printf( "FAILED - bluetooth error\n" );
		return PressAnyKeyToFinish( );
	}
	if (strncmp( szLine, "Disco", 5 ) ) 
	{
		printf( "FAILED - data error\n" );
		return PressAnyKeyToFinish( );
	}

	CloseHandle( ghOverlappedFile );
	printf( "OK\n" );

	// Start  absent device test
	printf( "Absent device connection (5 secs)..." );

	// Sleep 4 secs and continue - ToothPIC is trying to connect to a device that doesn't exist
	Sleep( 5000 );
	printf( "OK if no flashing red LED\n" );
			
	// Start  absent device test
	printf( "Device Enquiry (10 secs)............" );

	// Sleep 10 secs and continue - ToothPIC enquiring for devices
	Sleep( 10000 );
	printf( "OK if no flashing red LED\n" );
			
	// test authenticated connection with link key
	printf( "Connecting as paired slave.........." );
			
	// connect to com port
	ComStr.Format( "\\\\.\\COM%ld", IncomingCOM );
	if ( OpenOverlappedFile( ComStr ) )
	{
		printf( "Error can't open COM%ld:\n\n", IncomingCOM );
		return PressAnyKeyToFinish( );
	}

	// await greeting text
	if (ReadFromCom( szLine, 10 )) 
	{
		printf( "FAILED - bluetooth error\n" );
		return PressAnyKeyToFinish( );
	}
	if (strncmp( szLine, "PairMaster", 10 ) ) 
	{
		printf( "FAILED - data error\n" );
		return PressAnyKeyToFinish( );
	}

	printf( "OK\n" );

	// report number of discovered devices
	char NumDev;
	if (ReadFromCom( (char *)&NumDev, 1 ) )
	{
		printf( "FAILED - bluetooth error\n" );
		return PressAnyKeyToFinish( );
	}
	printf( "Number of devices found.............%ld\n" , (long) NumDev);
			
	// report device names
	char DevName[NAMEMAXLEN];
	for (int i = 0; i<NumDev; i++)
	{
		if (ReadFromCom( DevName, NAMEMAXLEN ) )
		{
			printf( "FAILED - bluetooth error\n" );
			return PressAnyKeyToFinish( );
		}
		printf( "Device %2ld...........................%s\n" , (long)i+1, DevName);
	}

#define COUNTDOWNFROM  6251

	// test bulk transfer in
	printf( "Testing 100kbit Tx from BlueMatik..." );
    struct __timeb64 tbStart, tbEnd;
	for (isCount = COUNTDOWNFROM; isCount!=0; isCount--)  	
	{
		if (isCount==COUNTDOWNFROM) _ftime64( &tbStart );
		if (isCount==1) _ftime64( &tbEnd );

		// assumes ToothPICTest is compiled little-endian
		if (ReadFromCom( (char *)&isReadVal, 2 ) )
		{
			printf( "FAILED - bluetooth error\n" );
			return PressAnyKeyToFinish( );
		}
		if ( isReadVal != isCount ) 
		{
			printf( "FAILED - data error\n" );
			return PressAnyKeyToFinish( );
		}
	} 		

	int bitrate = (int)(8 * (COUNTDOWNFROM-1) * 1000 / ((tbEnd.time - tbStart.time)*1000 + tbEnd.millitm - tbStart.millitm));
	printf( "%ld bits/s\n", bitrate );

	// test bulk transfer out
	_ftime64( &tbStart );
	printf( "Testing 100kbit Tx to BlueMatik....." );
	for (isCount = COUNTDOWNFROM; isCount!=0; isCount--)  	
	{
		// assumes ToothPICTest in compiled little-endian
		if (WriteToCom( (char *)&isCount, 2 ) )
		{
			printf( "FAILED - bluetooth error\n" );
			return PressAnyKeyToFinish( );
		}
	} 		
	// get 'OK'
	if (ReadFromCom( (char *)szLine, 2 ) )
	{
		printf( "FAILED - bluetooth error\n" );
		return PressAnyKeyToFinish( );
	}
	_ftime64( &tbEnd );
	if (strncmp( szLine, "OK", 2 ) ) 
	{
		printf( "FAILED - data error\n" );
		return PressAnyKeyToFinish( );
	}

	bitrate = (int)(8 * (COUNTDOWNFROM) * 1000 / ((tbEnd.time - tbStart.time)*1000 + tbEnd.millitm - tbStart.millitm));
	printf( ">%ld bits/s\n", bitrate );


	// get signal strength
	printf( "Enquiring Signal Strength..........." );

	// await phrase "Signal" and result byte
	if (ReadFromCom( szLine, 7 )) 
	{
		printf( "FAILED - bluetooth error\n" );
		return PressAnyKeyToFinish( );
	}
	if (strncmp( szLine, "Signal", 6 ) ) 
	{
		printf( "FAILED - data error\n" );
		return PressAnyKeyToFinish( );
	}

	// report result
	char Qual = szLine[6];
	if ( Qual==0 )  
		printf( "Within Golder Receiver Range\n" );
	else if (Qual > 0)
		printf( "%lddB too high\n", (long)Qual );
	else
		printf( "too low (%ld)\n", (long)Qual );

	// get link quality
	printf( "Enquiring Link Quality.............." );

	// await phrase "LinkQ" and result byte
	if (ReadFromCom( szLine, 6 )) 
	{
		printf( "FAILED - bluetooth error\n" );
		return PressAnyKeyToFinish( );
	}
	if (strncmp( szLine, "LinkQ", 5 ) ) 
	{
		printf( "FAILED - data error\n" );
		return PressAnyKeyToFinish( );
	}

	// calculate Bit Error Rate
	unsigned char BERval = szLine[5];
	double BER;
	if ( BERval >=215 )  
		BER = (255 - BERval) * 0.0025;
	else if ( BERval >=89 )  
		BER = 0.1 + (215 - BERval) * 0.08;
	else  
		BER = 10.18 + (89 - BERval) * 0.64;

	// report result
	printf( "Bit Error Rate = %.4lf%%\n", BER );

    // remote device disconnect test 
	printf( "\n\nPlease disconnect serial connection. \n" );
	printf( "Then ToothPIC will pulse the green light\nto indicate remote device disconnected\nsuccessfully.\n\n" );
			
	CloseHandle( ghOverlappedFile );

	PressAnyKeyToFinish( true );
	return 0;
}



int PressAnyKeyToFinish( bool bSuccess )
{
	if (!bSuccess)
		printf( "\nToothPIC Test FAILED\n" );	
	else
		printf( "\nToothPIC Test SUCEEDED\n" );	
	printf( "\nPress any key to finish\n" );	
	getchar();	
	getchar();	

	return ( bSuccess ? 0 : 1 );
}


// overlapped I/O routines
// if you're not familiar with overlapped, I/O, what is does is accept bytes irrespective
// of when bytes are transmitted - useful for event drive messaging such as FlexiPanel
// where you don't know who's going to initiate the next transmission and indeed both might 
// at the same time

int OpenOverlappedFile(CString& Filename)
{
	ghOverlappedFile = CreateFile (Filename,GENERIC_READ | GENERIC_WRITE,0,NULL,OPEN_EXISTING,
					FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,NULL); 

	if (ghOverlappedFile == INVALID_HANDLE_VALUE )
	{
		return GetLastError();
	}

	// set timeouts to 60 seconds.  
	COMMTIMEOUTS  cto;
	cto.ReadIntervalTimeout = 0;
	cto.ReadTotalTimeoutConstant = 60000;
	cto.ReadTotalTimeoutMultiplier = 0;
	cto.WriteTotalTimeoutConstant = 0;
	cto.WriteTotalTimeoutMultiplier = 60000;
	SetCommTimeouts( ghOverlappedFile, &cto );

	// Purge any rubbish stuck in buffer
	PurgeComm( ghOverlappedFile, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR );
	ZeroMemory( &gReadOverlapped, sizeof(gReadOverlapped) );
	ZeroMemory( &gWriteOverlapped, sizeof(gWriteOverlapped) );

	return 0;
}

int WriteToCom( char * lpBuffer, int nBytes )
{
	DWORD nBytesSent = 0xFFFFFFFF;

	if (nBytes==-1) nBytes = (int)strlen( lpBuffer );  // note, does not include zero terminator

	if (!WriteFile( ghOverlappedFile, lpBuffer, nBytes, &nBytesSent, &gWriteOverlapped ))  
	{
		int err = GetLastError();
		if ( err!=ERROR_IO_PENDING )  
		{
			return err;
		}
		else
		{
			while (!HasOverlappedIoCompleted(&gWriteOverlapped))  
			{	
				Sleep(10); 
			}

		}
	}

	return 0;

	/*
	DWORD NumWritten = 0;
	if (WriteFile( ghOverlappedFile, lpBuffer, (DWORD)strlen(lpBuffer), &NumWritten, NULL )) return true;

	// failed
	printf( "\n\nWrite error (bad COM port?)\n\n" );
	PressAnyKeyToFinish();
	return false;
	*/
}

int ReadFromCom( char * lpBuffer, DWORD iChars)
{
	DWORD NumReadCumulative = 0;
	DWORD NumRead = 0;

	int Result = ReadFile( ghOverlappedFile, ((char*)lpBuffer)+NumReadCumulative, 
						iChars - NumReadCumulative, &NumRead, &gReadOverlapped );
	NumReadCumulative += NumRead;
	if (!Result ) 
	{
		int err = GetLastError();
		if (err==ERROR_IO_PENDING)
		{
			while (!HasOverlappedIoCompleted(&gReadOverlapped))  
			{	
				Sleep(10); 
			}
		}
		else
		{
			return err;
		}
	}

/*	while ( NumReadCumulative < iChars ) 
	{
		if (!ReadFile( hFile, lpBuffer+NumReadCumulative, iChars-NumReadCumulative, &NumRead, NULL ))
		{
			printf( "\n\nRead error (bad COM port?)\n\n" );
			PressAnyKeyToFinish( );
			return false;
		}
		NumReadCumulative += NumRead;
	}
*/
	return 0;
}

