//------------------------------------------------------------------------------------------------------
//
//  HelloWorldBit.c
//
//  Hello World Bitstream tutorial
//
//  (c) FlexiPanel Ltd
//
//-------------------------------------------------------------------------------------------------------

#include "..\ToothPIC.h" 
#include <p18f6720.h>
 
rom unsigned char * szHello = "Hello\r\n";
rom unsigned char * szWorld = "World\r\n";

Bool InSlaveModeSlave = False;  // false if we need to send a BMTC_Slave command

void main( void )
{
	overlay long lVal;
	overlay short Row, Col, sVal;
	overlay char cVal;

	// if no BlueMatik, flash red led rapidly
	while ((ToothPICSemaphores&TPSF_BMTEXISTS)==0)
	{
		LedRed = ~LedRed;
		msDelay(50);
	}

	// main loop
	while ( 1 ) 
	{
		if (!InSlaveModeSlave)
		{
			BMTCommand( BMTC_Slave, 0, 0 );		// Start BlueMatik binary service
			AwaitBMTOK; 				
			InSlaveModeSlave = True;
		}

		// Flash LED to show we're alive
		LedGreen = ~LedGreen;
		msDelay( 250 );

	}  // end of while ( 1 ) 

}

// HighInterrupt handler
void HighInterrupt (void)
{
}

// LowInterrupt handler
void LowInterrupt (void)
{
	if (IsSWI( SWI_BMTData ) )
	{
		unsigned char ch = *BMTRxCh;

		// if the received character is an 'H', say hello
		if (ch=='h' || ch=='H')
		{
			BMTTransmit( szHello, 0, 7, 255 );
		}

		// if the received character is an 'W', say world
		if (ch=='w' || ch=='W')
		{
			BMTTransmit( szWorld, 0, 7, 255 );
		}

		// remove the character from the receive buffer
		BMTRxAdvanceCh;

		ClearSWI( SWI_BMTData );		// Clear interrupt flag
		return;
	}

	if (IsSWI( SWI_Tick ) )
	{
		ClearSWI( SWI_Tick );		// Clear interrupt flag
		return;
	}

}


// Error event handler 
void ErrorStatus( unsigned char ErrNum )
{
	// Flash error number for diagnositic purposes
	Breakpoint( ErrNum );
}


// BlueMatik Event handler 
void BMTEvent( unsigned char EventID, void *pData1, void *pData2 )
{
	if ( EventID==BMTE_Error || EventID==BMTE_Warning )
	{
		// Flash error number for diagnositic purposes
		Breakpoint( ((unsigned char*)pData1)[0] );
	}

	if ( EventID==BMTE_Connect ) 
	{
		// turn on  red led during connection
		LedRed = LedRedOn;
	}

	if ( EventID==BMTE_Disco ) 
	{
		// turn off red led after disconnection 
		LedRed = LedRedOff;

		// re-enter slave mode
		InSlaveModeSlave = False;
	}

}

// FxPEvent handler 
void FxPEvent( unsigned char EventID, void *pData )
{
}

